package org.dromara.system.controller.system;

import java.util.List;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.dromara.common.idempotent.annotation.RepeatSubmit;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.AddGroup;
import org.dromara.common.core.validate.EditGroup;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.excel.utils.ExcelUtil;
import org.dromara.system.domain.vo.CaseCommentSettingVo;
import org.dromara.system.domain.bo.CaseCommentSettingBo;
import org.dromara.system.service.ICaseCommentSettingService;
import org.dromara.common.mybatis.core.page.TableDataInfo;

/**
 * 病例点评配置
 *
 * @author wuzhigang
 * @date 2024-01-03
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/system/commentSetting")
public class CaseCommentSettingController extends BaseController {

    private final ICaseCommentSettingService caseCommentSettingService;

    /**
     * 查询病例点评配置列表
     */
    @GetMapping("/list")
    public TableDataInfo<CaseCommentSettingVo> list(CaseCommentSettingBo bo, PageQuery pageQuery) {
        return caseCommentSettingService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出病例点评配置列表
     */
    @Log(title = "病例点评配置", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(CaseCommentSettingBo bo, HttpServletResponse response) {
        List<CaseCommentSettingVo> list = caseCommentSettingService.queryList(bo);
        ExcelUtil.exportExcel(list, "病例点评配置", CaseCommentSettingVo.class, response);
    }

    /**
     * 获取病例点评配置详细信息
     *
     * @param id 主键
     */
    @GetMapping("/{id}")
    public R<CaseCommentSettingVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(caseCommentSettingService.queryById(id));
    }

    /**
     * 新增病例点评配置
     */
    @Log(title = "病例点评配置", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody CaseCommentSettingBo bo) {
        return toAjax(caseCommentSettingService.insertByBo(bo));
    }

    /**
     * 修改病例点评配置
     */
    @Log(title = "病例点评配置", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody CaseCommentSettingBo bo) {
        return toAjax(caseCommentSettingService.updateByBo(bo));
    }

    /**
     * 删除病例点评配置
     *
     * @param ids 主键串
     */
    @Log(title = "病例点评配置", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(caseCommentSettingService.deleteWithValidByIds(List.of(ids), true));
    }
}
